/* InsertField.groovy
October 2008 by John Green

Builds analysis data for insertion of a new field into every table
and every index for purposes of database consolidation.

--- USAGE ---
o  You need to dump your project's schema and propath into
   your working directory "prorefactor/projects/...". See:
   http://www.oehive.org/prorefactor/proj_cfg_dump

o  You'll need proparse.jar and 'scripts' in your CLASSPATH.

o  Configure insertfield.properties in your current working directory.

o  Run the script: groovy scripts/InsertField.groovy
-------------

*/

import org.prorefactor.refactor.RefactorSession
import org.prorefactor.treeparser.ParseUnit
import org.prorefactor.refactor.FileTargetComparator
import jdbm.btree.BTree
import jdbm.RecordManager
import jdbm.helper.Tuple
import org.prorefactor.refactor.FileTarget
import org.apache.commons.io.FileUtils

class InsertField implements Runnable {

	BTree targetTable
	private File dbDir
	Logger logger
	RecordManager recordManager
	RefactorSession prsession = RefactorSession.instance
	ScriptSupport support
	String relativePath
	HashSet<String> doneFileNames = new HashSet<String>()



	public static void main(String [] args) {
		new InsertField().run()
	}


	void run() {
		prsession.setProjectBinariesEnabledOff();
		support = new ScriptSupport('insertfield.properties', 'insertfield.log')
		support.hasPrepend = false
		dbInit()
		try {
			if (! support.initialize())
				return
			run2_fileList()
		} finally {
			support.close()
			dbClose()
		}
	}


	private void dbClose() throws IOException {
		if (recordManager!=null)
			recordManager.close();
		if (dbDir!=null && dbDir.exists())
			FileUtils.deleteDirectory(dbDir);
	}


	private void dbInit() throws IOException {
		dbDir = File.createTempFile("insertfield", null);
		dbDir.delete();
		dbDir.mkdir();
		String dbname = dbDir.getPath() + "/targets";
		Properties props = new Properties();
        props.put(jdbm.RecordManagerOptions.CACHE_SIZE, "1000" );
		props.put(jdbm.RecordManagerOptions.DISABLE_TRANSACTIONS, "true");
		recordManager = jdbm.RecordManagerFactory.createRecordManager(dbname, props);
		targetTable = BTree.createInstance(recordManager, new FileTargetComparator());
		recordManager.commit();
	}


	void run2_fileList() {
		logger = support.logger

		// Loop all compile units from the selection.
		for (File file in support.fileScanner) {
			relativePath = support.relativePath(file)
			logger.log(relativePath + ' ')
			def parseUnit = new ParseUnit(file)
			try {
				parseUnit.treeParser01()
			} catch (Exception e) {
				// Show and log parse errors.
				logger.logLine(e.message)
				continue
			}
			def converter = new InsertFieldUnit(parseUnit, support, targetTable, doneFileNames)
			converter.run()
			recordManager.commit()
			logger.newline()
		}

		logger.logLine("### Change can be automated:")
		logger.logLine("### file:line:column description")
		def browser = targetTable.browse();
		def tuple = new Tuple();
		def counter = 0
		while (browser.getNext(tuple)) {
			def conversion = (Conversion) tuple.getValue();
			if (conversion.preprocessorProblems)
				continue
			counter++
			def target = (FileTarget) tuple.getKey();
			logger.logLine(
					target.filename + ':' + target.line + ':' + target.column
					+ '  ' + conversion.reportText
					)
		}

		logger.newline()
		logger.logLine("### Manual change required:")
		logger.logLine("### file:line:column description")
		browser = targetTable.browse();
		def counter2 = 0
		while (browser.getNext(tuple)) {
			def conversion = (Conversion) tuple.getValue();
			if (! conversion.preprocessorProblems)
				continue
			counter2++
			def target = (FileTarget) tuple.getKey();
			logger.logLine(
					target.filename + ':' + target.line + ':' + target.column
					+ '  ' + conversion.reportText
					)
		}
		logger.newline()
		logger.logLine("### CAN auto-change $counter")
		logger.logLine("### Can NOT auto-change $counter2")

	}


}
